import java.io.*;
import java.util.Vector;

public class Cons {

  // See well_formed() method below.

  String label;
  Cons head;
  Cons tail;

  // Cons constructors

  public Cons (String s) {
    label = s;
  }  // Cons constructor (leaf)

  public Cons (Cons head, Cons tail) {
    this.head = head;
    this.tail = tail;
  }  // Cons constructor (pair)

  public Cons (Cons element) {
    this.head = element;
    this.tail = new Cons("");
  }  // Cons constructor (1-element list, not really necessary)

  public boolean well_formed() {
    if (head == null || tail == null)
      return (label != null && head == null && tail == null);
    else
      return (label == null && head.well_formed() && tail.well_formed());
  }  // well-formed

  public boolean leaf() {
    return (head == null || tail == null);
  }  // leaf

  public boolean end() {
    return leaf();
  }  // end

  public boolean list() {
    if (leaf())
      return label.equals("");
    else
      return tail.list();
  }  // list

  public Cons append(Cons c) {
    if (leaf())
      return c;
    else
      return new Cons(head, tail.append(c));
  }  // append

  public Cons reverse() {
    if (leaf())
      return this;
    else
      return tail.reverse().append(new Cons(head));
  }  // reverse

  public int length() {
    if (leaf())
      return 0;
    else
      return (1 + tail.length());
  }  // length

  public Cons ith_member(int i) {  //
    Cons c = this;
    while (i > 0 && !c.leaf()) {
      i--;
      c = c.tail;
    }
    return (c.leaf() ? null : c.head);
  }  // ith_member (counts from 0, because this is a C-ish language :-)

  private void to_vec(Vector v) throws NumberFormatException {
    if (!leaf()) {
      v.addElement(new Integer(head.label));
      tail.to_vec(v);
    }
  }  // to_vec

  public Vector to_vector() throws NumberFormatException {
    Vector v = new Vector(length());
    to_vec(v);
    return v;
  }  // to_vector

  public Cons sub_at_pos(Vector pv) {
    Cons c = this;
    for (int i = 0; i < pv.size(); i++)
      c = c.ith_member(((Integer) pv.elementAt(i)).intValue());
    return c;
  }  // sub_at_pos

  public int hashCode() {
    if (leaf())
      return label.hashCode();  // string hashCode()
    else
      return head.hashCode() ^ (tail.hashCode()<<1);
  }  // hashCode

  public boolean equals(Cons c) {
    // assume both are well-formed
    if (leaf() || c.leaf())
      return (leaf() && c.leaf());
    else
      return (head.equals(c.head) && tail.equals(c.tail));
  }  // equals

  private Cons dotfix() {
    if (length() == 3 && tail.head.label.equals(".")) {
      return new Cons(head, tail.tail.head);
    }
    else
      return this;
  }  // dotfix

  public static Cons parse_list (Parser par) {
    String tok = par.getToken();

    if (tok.equals("")) {
      par.error = true;
      return null;
    }
    else if (tok.equals("(")) {
      boolean dot = false;
      Cons c = new Cons("");
      Cons c1 = Cons.parse_list(par);
      while (c1 != null) {
	c = c.append(new Cons(c1));
	if (c1.label != null && c1.label.equals("."))
	  dot = true;
	c1 = Cons.parse_list(par);
      }
      return (dot ? c.dotfix() : c);
    }
    else if (tok.equals(")"))
      return null;
    else
      return new Cons(tok);
  }  // parse_list method

  public static Cons build_cons_from_string(String str) throws Cons_exception {
    Parser par = new Parser();
    par.pos = 0;
    par.str = str;
    Cons c = Cons.parse_list(par);
    if (par.error)
      throw new Cons_exception("incomplete cons: " + par.str);
    else if (par.moreToParse()) {
      String junk = par.str.substring(par.pos, par.str.length());
      throw new Cons_exception("takens after end of cons: " + junk);
    }
    return c;
  }  // build_cons_from_string

  private void st_rec (StringBuffer sb) {
    if (list()) {
      // empty list ok
      sb.append("(");
      for (Cons c = this; !c.leaf(); c = c.tail) {
	if (c != this) sb.append(" ");
	c.head.st_rec(sb);
      }
      sb.append(")");
    }
    else if (leaf()) {
      sb.append(label);
    }
    else {
      sb.append("(");
      head.st_rec(sb);
      sb.append(" . ");
      tail.st_rec(sb);
      sb.append(")");
    }
  }  // st_rec

  public String toString () {
    StringBuffer sb = new StringBuffer();
    st_rec(sb);
    return sb.toString();
  }  // toString

  private static String read_string_from_file(String file) throws IOException {
    File f = new File(file);
    int size = (int) f.length();
    String str = "";

    FileInputStream in = new FileInputStream(f);
    byte[] data = new byte[size];
    int bytes_read = 0;
    while (bytes_read < size)
      bytes_read += in.read(data,bytes_read, size-bytes_read);
    in.close();
    str = new String(data, 0);
    return str;
  }  // read_string_from_file

  public static Cons get_list_from_file(String filename)
       throws IOException, Cons_exception {
    String s = read_string_from_file(filename);
    Cons c = Cons.build_cons_from_string(s);
    return c;
  }  // get_list_from_file

  public boolean neg_literal() {
    return (length() == 2 && head.label.equals("not"));
  }  // neg_literal

  public Cons atom() {
    return (neg_literal() ? tail.head : this);
  }  // atom

  public static void main(String args[])
       throws NumberFormatException, IOException, Cons_exception {
    if (args.length != 1) {
      System.out.println("need one argument");
      System.exit(-1);
    }
    Cons c = get_list_from_file(args[0]);
    System.out.println("term:    " + c.head);
    System.out.println("vec:     " + c.tail.head);
    Vector v = c.tail.head.to_vector();
    Cons sub = c.head.sub_at_pos(v);
    System.out.println("subterm: " + sub);
  }  // main
}  // Cons class

class Cons_exception extends Exception {
  public Cons_exception() { super(); }
  public Cons_exception(String s) { super(s); }
}  // Cons_exception
