import java.awt.*;

// Extension of Polygon with extra methods.
// Also, with ordinary Java Polygons, to make a closed polygon, the
// first point has to be the same as the last point.  Here, all
// polygons (>2) are considered to be closed.

class X_polygon extends Polygon {

  static int turn_calls = 0;

  private static final int LEFT                   = 8;
  private static final int RIGHT                  = 9;
  private static final int AHEAD                  = 5;
  private static final int BACK                   = 6;
  private static final int NONE                   = 7;
  
  public X_polygon() {
  }  // null constructor

  public X_polygon(int[] xp, int[] yp, int n) {
    super(xp, yp, n);
  }  // basic constructor, same as polygon

  public X_polygon extend(X_polygon r, int xoff, int yoff) {
    X_polygon p = new X_polygon();
    p.npoints = npoints + r.npoints;
    p.xpoints = new int[p.npoints];
    p.ypoints = new int[p.npoints];

    // We are building a new polygon which is a copy of "this" with all of r
    // inserted before the last point of "this".
    // Apply offsets xoff and yoff to r.

    // Copy all but last point of "this" to p.
    int i;
    for (i = 0; i < npoints-1; i++) {
      p.xpoints[i] = xpoints[i];
      p.ypoints[i] = ypoints[i];
    }
    // Add all points of r to p.
    for (int j = 0; j < r.npoints; j++) {
      p.xpoints[i] = r.xpoints[j] + xoff;
      p.ypoints[i] = r.ypoints[j] + yoff;
      i++;
    }
    // Add last point of "this" to p.
    p.xpoints[i] = xpoints[npoints-1];
    p.ypoints[i] = ypoints[npoints-1];
    i++;
    
    if (i != p.npoints) System.err.println("extend error");

    p.make_convex();
    return p;
  }  // extend

  public void offset(int xo, int yo) {
    for (int i = 0; i < npoints; i++) {
      xpoints[i] += xo;
      ypoints[i] += yo;
    }
  }  // offset

  public static double direction(int ax, int ay, int bx, int by) {
    return (Math.atan2((double) by-ay, (double) bx-ax));
  }  // direction

  public static double turn(int ax, int ay,
			    int bx, int by,
			    int cx, int cy,
			    int dx, int dy) {
    turn_calls++;

    // (0 < result < PI) => left turn (right in Java) not crossing left horizon
    // (-2PI < result < -PI) => left turn (rt. in Java) crossing left horizon

    // If either segment is just a point, there is no turn.
    if ((ax == bx && ay == by) || (cx == dx && cy == dy))
      return (double) 0;
    else {
      double t1 = direction(ax, ay, bx, by);
      double t2 = direction(cx, cy, dx, dy);
      // System.out.println("turn: t1=" + t1 + ", t2 =" + t2);
      return (t2-t1);
    }
  }  // turn

  public static int turn_type(int ax, int ay,
				int bx, int by,
				int cx, int cy,
				int dx, int dy) {
    // Left and right assume Java's upside-down coordinate system.
    if ((ax == bx && ay == by) || (cx == dx && cy == dy))
      return NONE;
    else {
      double t = turn(ax, ay, bx, by, cx, cy, dx, dy);
      if (t == 0.0 || Math.abs(t) == 2*Math.PI)
	return AHEAD;
      else if (Math.abs(t) == Math.PI)
	return BACK;
      else if (0.0 < t && t < Math.PI)
	return RIGHT;  // Doesn't cross left horizon
      else if (-Math.PI < t && t < 0.0)
	return LEFT;   // Doesn't cross left horizon
      else if (-2*Math.PI < t && t < -Math.PI)
	return RIGHT;
      else if (Math.PI < t && t < 2*Math.PI)
	return LEFT;
      else {
	System.err.println("turn_type: " + t);
	return -1;
      }
    }
  }  // turn_type

  private static boolean dir_ccw(double a, double b, double c) {
    // Assuming Java's upside-down coordinates.
    if (b < 0 && 0 < c)
      b += 2 * Math.PI;
    return (0 < a-b && a-b < a-c) || (a-b < a-c && a-c < 0);
  }  // dir_ccw

  private static boolean dir_cw(double a, double b, double c) {
    return dir_ccw(c, b, a);
  }  // dir_cw

  public boolean clockwise() {
    if (npoints < 3)
      return false;
    else {
      int ttx = turn_type(xpoints[npoints-1], ypoints[npoints-1],
			    xpoints[0], ypoints[0],
			    xpoints[0], ypoints[0],
			    xpoints[1], ypoints[1]);
      return (ttx == RIGHT);
    }
  }  // clockwise

  public boolean counter_clockwise() {
    if (npoints < 3)
      return false;
    else {
      int ttx = turn_type(xpoints[npoints-1], ypoints[npoints-1],
			    xpoints[0], ypoints[0],
			    xpoints[0], ypoints[0],
			    xpoints[1], ypoints[1]);
      return (ttx == LEFT);
    }
  }  // counter_clockwise

  public void make_convex() {
    if (npoints < 3)
      return;
    else {
      boolean cw = clockwise();

      double last_direction = direction(xpoints[npoints-1], ypoints[npoints-1],
					xpoints[0], ypoints[0]);
      boolean convex = false;
      int i = 0;
      while (!convex) {
	boolean ok_so_far = true;

	while (i < npoints-2 && ok_so_far){
	  double d1 = direction(xpoints[i],    ypoints[i],
				xpoints[i+1], ypoints[i+1]);
	  double d2 = direction(xpoints[i+1], ypoints[i+1],
				xpoints[i+2], ypoints[i+2]);
	  if (cw ? dir_cw(d1,d2,last_direction) : dir_ccw(d1,d2,last_direction))
	    i++;
	  else
	    ok_so_far = false;
	}
	if (ok_so_far)
	  convex = true;
	else {
	  // Delete point (i+1).
	  for (int j = i+1; j < npoints-1; j++) {
	    xpoints[j] = xpoints[j+1];
	    ypoints[j] = ypoints[j+1];
	  }
	  npoints--;
	  if (i > 0) 
	    i--;
	}
      }
    }
  }  // make_convex

  public boolean my_inside(int x, int y) {
    // True also if the point is on the perimeter.
    // Assume it is convex and clockwise.  Go around once, making
    // sure that the point is always ahead or on the correct side of
    // current segment.
    // System.out.println("My_inside: " + this + " " + x + " " + y);

    boolean ccw = counter_clockwise();

    for (int i = 0; i < npoints; i++) {
      int ttx = turn_type(xpoints[i],             ypoints[i],
			    xpoints[(i+1)%npoints], ypoints[(i+1)%npoints],
			    xpoints[i],             ypoints[i],
			    x,                      y);

      if (ccw) {
	if (!(ttx == AHEAD || ttx == LEFT))
	  return false;
      }
      else {
	if (!(ttx == AHEAD || ttx == RIGHT))
	  return false;
      }
    }
    return true;
  }  // my_inside

  public static boolean between(int a, int b, int c) {
    if (a < c)
      return (a <= b && b <= c);
    else
      return (c <= b && b <= a);
  }  // between

  private static boolean opposite_turns(int t1, int t2) {
    switch (t1) {
    case RIGHT: return (t2 == LEFT || neither(t2));
    case LEFT:  return (t2 == RIGHT || neither(t2));
    default:    return (t2 == RIGHT || t2 == LEFT);
    }
  }  // opposite_turns

  private static boolean neither(int t) {
    return (t == NONE || t == AHEAD || t == BACK);
  }  // neither

  public static boolean segs_intersect(int ax, int ay, int bx, int by,
				       int cx, int cy, int dx, int dy) {
    int t1 = turn_type(ax, ay, bx, by, ax, ay, cx, cy);
    int t2 = turn_type(ax, ay, bx, by, ax, ay, dx, dy);
    int t3 = turn_type(cx, cy, dx, dy, cx, cy, ax, ay);
    int t4 = turn_type(cx, cy, dx, dy, cx, cy, bx, by);

    if (opposite_turns(t1, t2) && opposite_turns(t3, t4))
      return true;
    else if (neither(t1) && neither(t2) && neither(t3) && neither(t4))
      // All 4 points are collinear.
      return (between(ax, cx, bx) || between(ax, dx, bx));
    else
      return  false;
  }  // segs_intersect
  
  public static boolean overlap(X_polygon a, X_polygon b, int xoff, int yoff) {
    // Assume convex polygons.
    // Offsets are applied to polygon b.

    // Check if any segments intersect.
    for (int i = 0; i < a.npoints; i++) {
      int px = a.xpoints[i];
      int py = a.ypoints[i];
      int qx = a.xpoints[(i+1) % a.npoints];
      int qy = a.ypoints[(i+1) % a.npoints];
      for (int j = 0; j < b.npoints; j++) {
	int rx = b.xpoints[j] + xoff;
	int ry = b.ypoints[j] + yoff;
	int sx = b.xpoints[(j+1) % b.npoints] + xoff;
	int sy = b.ypoints[(j+1) % b.npoints] + yoff;
	if (segs_intersect(px, py, qx, qy, rx, ry, sx, sy))
	  return true;
      }
    }
    // Make sure that neither is completely inside the other.
    // Since no segments intersect, we need to check only one point for each.
    if (a.my_inside(b.xpoints[0] + xoff, b.ypoints[0] + yoff))
      return true;
    else if (b.my_inside(a.xpoints[0] - xoff, a.ypoints[0] - yoff))
      return true;
    else
      return false;
  }  // overlap

  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append(npoints);
    for (int i = 0; i < npoints; i++)
      sb.append(" (" + xpoints[i] + "," + ypoints[i] + ")");
    return sb.toString();
  } // toString

  public static void main(String[] args) {
    int x0 = (new Integer(args[0])).intValue();
    int y0 = (new Integer(args[1])).intValue();
    int x1 = (new Integer(args[2])).intValue();
    int y1 = (new Integer(args[3])).intValue();
    int x2 = (new Integer(args[4])).intValue();
    int y2 = (new Integer(args[5])).intValue();
    int x3 = (new Integer(args[6])).intValue();
    int y3 = (new Integer(args[7])).intValue();
    int x4 = (new Integer(args[8])).intValue();
    int y4 = (new Integer(args[9])).intValue();
    int x5 = (new Integer(args[10])).intValue();
    int y5 = (new Integer(args[11])).intValue();

    int[] ax = {x0, x1, x2};
    int[] ay = {y0, y1, y2};

    int[] bx = {x3, x4, x5};
    int[] by = {y3, y4, y5};

    X_polygon a = new X_polygon(ax, ay, 3);
    X_polygon b = new X_polygon(bx, by, 3);

    boolean o1 = overlap(a, b, 0, 0);
    boolean o2 = overlap(b, a, 0, 0);

    boolean in = a.my_inside(1,1);	

    System.out.println(a);	
    System.out.println(o1 + " " + o2 + " " + in);
  }

}  // class X_polygon
